@@ -32,6 +32,15 @@ setClassUnion("GenomicRangesORmissing", c("GenomicRanges", "missing"))
 minStartPerGRangesSequence <- function(x)
 {
     cil <- splitAsList(start(x), seqnames(x))  # CompressedIntegerList object
+    ## The 4 lines below are equivalent to:
+    ##   ans <- min(cil)
+    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ## but much faster!
+    ## TODO: Replace with the above, but only when the "min" method for
+    ## CompressedIntegerList objects (implemented in the IRanges package)
+    ## is as fast as the "viewMins" method for XIntegerViews objects
+    ## (implemented in C in the XVector package). Ideally, the 2 methods
+    ## should share the same underlying code.
     v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
     ans <- viewMins(v)
     ans[width(v) == 0L] <- NA_integer_
@@ -42,6 +51,15 @@ minStartPerGRangesSequence <- function(x)
 maxEndPerGRangesSequence <- function(x)
 {
     cil <- splitAsList(end(x), seqnames(x))  # CompressedIntegerList object
+    ## The 4 lines below are equivalent to:
+    ##   ans <- max(cil)
+    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ## but much faster!
+    ## TODO: Replace with the above, but only when the "max" method for
+    ## CompressedIntegerList objects (implemented in the IRanges package)
+    ## is as fast as the "viewMaxs" method for XIntegerViews objects
+    ## (implemented in C in the XVector package). Ideally, the 2 methods
+    ## should share the same underlying code.
     v <- Views(cil@unlistData, cil@partitioning)  # XIntegerViews object
     ans <- viewMaxs(v)
     ans[width(v) == 0L] <- NA_integer_
@@ -78,6 +96,10 @@ setGeneric("extraColumnSlotNames",
 
 setMethod("extraColumnSlotNames", "ANY", function(x) character())
 
+setMethod("fixedColumnNames", "GenomicRanges", function(x) {
+  colnames(as.data.frame(new(class(x))))
+})
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
 ###
@@ -405,6 +427,10 @@ setReplaceMethod("seqinfo", "GenomicRanges",
 )
 
 setMethod("score", "GenomicRanges", function(x) mcols(x)$score)
+setReplaceMethod("score", "GenomicRanges", function(x, value) {
+  mcols(x)$score <- value
+  x
+})
 
 #setReplaceMethod("constraint", "GenomicRanges",
 #    function(x, value)
@@ -519,9 +545,9 @@ setReplaceMethod("width", "GenomicRanges",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting and combining.
+### Subsetting.
 ###
-        
+
 setMethod(IRanges:::extractROWS, "GenomicRanges",
     function(x, i)
     {
@@ -632,6 +658,87 @@ setReplaceMethod("[", "GenomicRanges",
     }
 )
 
+### Subsetting a named list-like object *by* a GenomicRanges subscript.
+### The returned object has the class of 'x' and is parallel to 'gr'.
+.subset_by_GenomicRanges <- function(x, gr)
+{
+    x_names <- names(x)
+    if (!(is.list(x) || is(x, "List")) || is.null(x_names))
+        stop("subsetting by a GenomicRanges subscript only works ",
+             "on a list-like object with names")
+    irl <- split(ranges(gr), seqnames(gr), drop=TRUE)
+    seqlevels_in_use <- names(irl)
+    seqlevels2names <- match(seqlevels_in_use, x_names)
+    if (any(is.na(seqlevels2names)))
+        stop("subsetting by a GenomicRanges subscript, the names of the ",
+             "object to subset must contain the seqnames of the subscript")
+    tmp <- lapply(seq_along(seqlevels_in_use),
+                  function(i) {
+                      seqlevel <- seqlevels_in_use[i]
+                      name <- x_names[seqlevels2names[i]]
+                      extractList(x[[name]], irl[[seqlevel]])
+                  })
+
+    ## Unsplit 'tmp'.
+    ans <- as(do.call(c, tmp), class(x))
+    ans_len <- length(gr)
+    idx <- unlist(split(seq_len(ans_len), seqnames(gr), drop=TRUE))
+    revidx <- integer(ans_len)
+    revidx[idx] <- seq_len(ans_len)
+    names(ans) <- names(idx)
+    ans <- ans[revidx]
+    ans
+}
+
+setMethod("[", c("List", "GenomicRanges"),
+    function(x, i, j, ..., drop=TRUE)
+    {
+        if (!missing(j) || length(list(...)) > 0L)
+            stop("invalid subsetting")
+        .subset_by_GenomicRanges(x, i)
+    }
+)
+
+setMethod("[", c("list", "GenomicRanges"),
+    function(x, i, j, ..., drop=TRUE)
+    {
+        if (!missing(j) || length(list(...)) > 0L)
+            stop("invalid subsetting")
+        .subset_by_GenomicRanges(x, i)
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### $ and $<- methods
+###
+### Provided as a convenience, for GenomicRanges *only*, and as the result
+### of strong popular demand.
+### Note that those methods are not consistent with the other $ and $<-
+### methods in the IRanges/GenomicRanges infrastructure, and might confuse
+### some users by making them believe that a GenomicRanges object can be
+### manipulated as a data.frame-like object.
+### Therefore we recommend using them only interactively, and we discourage
+### their use in scripts or packages. For the latter, use 'mcols(x)$name'
+### instead of 'x$name'.
+###
+
+.DollarNames.GenomicRanges <- function(x, pattern)
+    grep(pattern, names(mcols(x)), value=TRUE)
+
+setMethod("$", "GenomicRanges",
+    function(x, name) mcols(x)[[name]]
+)
+
+setReplaceMethod("$", "GenomicRanges",
+    function(x, name, value) {mcols(x)[[name]] <- value; x}
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Combining.
+###
+
 ### Not exported. 'x' *must* be an unnamed list of length >= 1 (not checked).
 .unlist_list_of_GenomicRanges <- function(x, ignore.mcols=FALSE)
 {
@@ -672,32 +779,6 @@ setMethod("c", "GenomicRanges",
 )
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### $ and $<- methods
-###
-### Provided as a convenience, for GenomicRanges *only*, and as the result
-### of strong popular demand.
-### Note that those methods are not consistent with the other $ and $<-
-### methods in the IRanges/GenomicRanges infrastructure, and might confuse
-### some users by making them believe that a GenomicRanges object can be
-### manipulated as a data.frame-like object.
-### Therefore we recommend using them only interactively, and we discourage
-### their use in scripts or packages. For the latter, use 'mcols(x)$name'
-### instead of 'x$name'.
-###
-
-.DollarNames.GenomicRanges <- function(x, pattern)
-    grep(pattern, names(mcols(x)), value=TRUE)
-
-setMethod("$", "GenomicRanges",
-    function(x, name) mcols(x)[[name]]
-)
-
-setReplaceMethod("$", "GenomicRanges",
-    function(x, name, value) {mcols(x)[[name]] <- value; x}
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### "show" method.
 ###
@@ -707,7 +788,7 @@ setReplaceMethod("$", "GenomicRanges",
     lx <- length(x)
     nc <- ncol(mcols(x))
     ans <- cbind(seqnames=as.character(seqnames(x)),
-                 ranges=IRanges:::showAsCell(ranges(x)),
+                 ranges=showAsCell(ranges(x)),
                  strand=as.character(strand(x)))
     extraColumnNames <- extraColumnSlotNames(x)
     if (length(extraColumnNames) > 0L) {
@@ -715,8 +796,7 @@ setReplaceMethod("$", "GenomicRanges",
                        c(list(ans), lapply(extraColumnSlots(x), showAsCell)))
     }
     if (nc > 0L) {
-        tmp <- do.call(data.frame, c(lapply(mcols(x),
-                                            IRanges:::showAsCell),
+        tmp <- do.call(data.frame, c(lapply(mcols(x), showAsCell),
                                      list(check.names=FALSE)))
         ans <- cbind(ans, `|`=rep.int("|", lx), as.matrix(tmp))
     }
