@@ -24,9 +24,6 @@
     s_len <- length(subject)
     if (q_len == 0L || s_len == 0L)
         return(new("Hits", queryLength=q_len, subjectLength=s_len))
-    if (type != "any")
-        stop("overlap type \"", type, "\" is not yet supported ",
-             "for circular sequence ", names(circle.length))
     subject0 <- .putRangesOnFirstCircle(subject, circle.length)
     inttree0 <- IntervalTree(subject0)
     query0 <- .putRangesOnFirstCircle(query, circle.length)
@@ -178,8 +175,13 @@ setMethod("findOverlaps", c("GenomicRanges", "GenomicRanges"),
     f <- paste(by[,1], by[,2], sep="|")
     f <- factor(f, levels=unique(f))
     cil <- splitAsList(x, f)  # CompressedIntegerList
-    v <- Views(cil@unlistData, cil@partitioning)
-    viewSums(v)
+    ## The line below is equivalent to 'sum(unname(cil))', but much faster!
+    ## TODO: Replace with 'sum(unname(cil))', but only when the "sum" method
+    ## for CompressedIntegerList objects (implemented in the IRanges package)
+    ## is as fast as the "viewSums" method for XIntegerViews objects
+    ## (implemented in C in the XVector package). Ideally, the 2 methods
+    ## should share the same underlying code.
+    viewSums(Views(cil@unlistData, cil@partitioning))
 }
 
 .updateMatchMatrix <- function(matchMatrix, intrsct, minoverlap) {
@@ -499,113 +501,11 @@ setMethod("findOverlaps", c("GRangesList", "RangedData"),
     }
 )
 
-setMethod("findOverlaps", c("GAlignments", "Vector"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(grglist(query), subject,
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("Vector", "GAlignments"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(query, grglist(subject),
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-### Not strictly needed! Defining the above 2 methods covers that case but
-### with the following note:
-###   > findOverlaps(al1, al0)
-###   Note: Method with signature "GAlignments#ANY" chosen for
-###    function "findOverlaps", target signature
-###    "GAlignments#GAlignments".
-###    "ANY#GAlignments" would also be valid
-setMethod("findOverlaps", c("GAlignments", "GAlignments"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(grglist(query), grglist(subject),
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("GAlignments", "GRangesList"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(grglist(query), subject,
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("GRangesList", "GAlignments"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(query, grglist(subject),
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("GAlignmentPairs", "Vector"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(as(query, "GRangesList"), subject,
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("Vector", "GAlignmentPairs"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(query, as(subject, "GRangesList"),
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
-setMethod("findOverlaps", c("GAlignmentPairs", "GAlignmentPairs"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        findOverlaps(as(query, "GRangesList"), as(subject, "GRangesList"),
-                     maxgap = maxgap, minoverlap = minoverlap,
-                     type = match.arg(type), select = match.arg(select),
-                     ignore.strand = ignore.strand)
-    }
-)
-
 setMethod("findOverlaps", c("SummarizedExperiment", "Vector"),
     function(query, subject, maxgap = 0L, minoverlap = 1L,
              type = c("any", "start", "end", "within", "equal"),
-             select = c("all", "first"), ignore.strand = FALSE)
+             select = c("all", "first", "last", "arbitrary"),
+             ignore.strand = FALSE)
     {
         findOverlaps(rowData(query), subject,
                      maxgap = maxgap, minoverlap = minoverlap,
@@ -617,7 +517,8 @@ setMethod("findOverlaps", c("SummarizedExperiment", "Vector"),
 setMethod("findOverlaps", c("Vector", "SummarizedExperiment"),
     function(query, subject, maxgap = 0L, minoverlap = 1L,
              type = c("any", "start", "end", "within", "equal"),
-             select = c("all", "first"), ignore.strand = FALSE)
+             select = c("all", "first", "last", "arbitrary"),
+             ignore.strand = FALSE)
     {
         findOverlaps(query, rowData(subject),
                      maxgap = maxgap, minoverlap = minoverlap,
@@ -628,8 +529,9 @@ setMethod("findOverlaps", c("Vector", "SummarizedExperiment"),
 
 setMethod("findOverlaps", c("SummarizedExperiment", "SummarizedExperiment"),
     function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
+             type = c("any", "start", "end", "within", "equal"),
+             select = c("all", "first", "last", "arbitrary"),
+             ignore.strand = FALSE)
     {
         findOverlaps(rowData(query), rowData(subject),
                      maxgap = maxgap, minoverlap = minoverlap,
@@ -638,53 +540,12 @@ setMethod("findOverlaps", c("SummarizedExperiment", "SummarizedExperiment"),
     }
 )
 
-setMethod("findOverlaps", c("GAlignmentsList", "Vector"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        hits <- findOverlaps(grglist(unlist(query, use.names = FALSE)),
-                             subject, maxgap = maxgap, minoverlap = minoverlap,
-                             type = match.arg(type), select = match.arg(select),
-                             ignore.strand = ignore.strand)
-        remapHits(hits, query.map=factor(togroup(query)))
-    }
-)
-
-setMethod("findOverlaps", c("Vector", "GAlignmentsList"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        hits <- findOverlaps(query, grglist(unlist(subject, use.names = FALSE)),
-                             maxgap = maxgap, minoverlap = minoverlap,
-                             type = match.arg(type), select = match.arg(select),
-                             ignore.strand = ignore.strand)
-        remapHits(hits, subject.map=factor(togroup(subject)))
-    }
-)
-
-setMethod("findOverlaps", c("GAlignmentsList", "GAlignmentsList"),
-    function(query, subject, maxgap = 0L, minoverlap = 1L,
-             type = c("any", "start", "end", "within"),
-             select = c("all", "first"), ignore.strand = FALSE)
-    {
-        hits <- findOverlaps(grglist(unlist(query, use.names = FALSE)), 
-                             grglist(unlist(subject, use.names = FALSE)),
-                             maxgap = maxgap, minoverlap = minoverlap,
-                             type = match.arg(type), 
-                             select = match.arg(select),
-                             ignore.strand = ignore.strand)
-        remapHits(hits, subject.map=factor(togroup(subject)),
-                  query.map=factor(togroup(query)))
-    }
-)
 
 ### =========================================================================
 ### findOverlaps-based methods
 ### -------------------------------------------------------------------------
 
-.countOverlaps.definition <- function(query, subject,
+countOverlaps.definition <- function(query, subject,
         maxgap = 0L, minoverlap = 1L,
         type = c("any", "start", "end", "within", "equal"),
         ignore.strand = FALSE)
@@ -707,30 +568,14 @@ setMethod("findOverlaps", c("GAlignmentsList", "GAlignmentsList"),
     c("GRanges", "GRangesList"),
     c("GRangesList", "GRanges"),
 
-    c("GAlignments", "Vector"),
-    c("Vector", "GAlignments"),
-    c("GAlignments", "GAlignments"),
-    c("GAlignments", "GenomicRanges"),
-    c("GenomicRanges", "GAlignments"),
-    c("GAlignments", "GRangesList"),
-    c("GRangesList", "GAlignments"),
-
-    c("GAlignmentPairs", "Vector"),
-    c("Vector", "GAlignmentPairs"),
-    c("GAlignmentPairs", "GAlignmentPairs"),
-
-    c("GAlignmentsList", "Vector"),
-    c("Vector", "GAlignmentsList"),
-    c("GAlignmentsList", "GAlignmentsList"),
-
     c("SummarizedExperiment", "Vector"),
     c("Vector", "SummarizedExperiment"),
     c("SummarizedExperiment", "SummarizedExperiment")
 )
 
-setMethods("countOverlaps", .signatures1, .countOverlaps.definition)
+setMethods("countOverlaps", .signatures1, countOverlaps.definition)
 
-.overlapsAny.definition <- function(query, subject,
+overlapsAny.definition <- function(query, subject,
         maxgap = 0L, minoverlap = 1L,
         type = c("any", "start", "end", "within", "equal"),
         ignore.strand = FALSE)
@@ -742,7 +587,7 @@ setMethods("countOverlaps", .signatures1, .countOverlaps.definition)
                         ignore.strand = ignore.strand))
 }
 
-.subsetByOverlaps.definition1 <- function(query, subject,
+subsetByOverlaps.definition1 <- function(query, subject,
         maxgap = 0L, minoverlap = 1L,
         type = c("any", "start", "end", "within", "equal"), 
         ignore.strand = FALSE)
@@ -792,78 +637,18 @@ setMethods("countOverlaps", .signatures1, .countOverlaps.definition)
     c("RangedData", "GRangesList"),
     c("GenomicRanges", "RangedData"),
     c("GRangesList", "RangedData"),
-    c("GAlignments", "Vector"),
-    c("Vector", "GAlignments"),
-    c("GAlignments", "GAlignments"),
-    c("GAlignmentPairs", "Vector"),
-    c("Vector", "GAlignmentPairs"),
     c("SummarizedExperiment", "Vector"),
     c("Vector", "SummarizedExperiment"),
-    c("SummarizedExperiment", "SummarizedExperiment"),
-    c("GAlignmentsList", "Vector"),
-    c("Vector", "GAlignmentsList"),
-    c("GAlignmentsList", "GAlignmentsList")
+    c("SummarizedExperiment", "SummarizedExperiment")
 )
 
 for (sig in .signatures2) {
-    setMethod("overlapsAny", sig, .overlapsAny.definition)
+    setMethod("overlapsAny", sig, overlapsAny.definition)
     if (sig[1L] == "RangesList")
         setMethod("subsetByOverlaps", sig, .subsetByOverlaps.definition2)
     else if (sig[1L] %in% c("RangedData", "SummarizedExperiment"))
         setMethod("subsetByOverlaps", sig, .subsetByOverlaps.definition3)
     else
-        setMethod("subsetByOverlaps", sig, .subsetByOverlaps.definition1)
+        setMethod("subsetByOverlaps", sig, subsetByOverlaps.definition1)
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### match(), %in%, findMatches(), and countMatches()
-###
-### All "match" and "%in% methods (except methods for GenomicRanges objects)
-### are defunct in BioC 2.13.
-### Methods for GenomicRanges have been moved to GenomicRanges-comparison.R
-### and their behavior changed to do equality instead of overlaps.
-
-.signatures3 <- .signatures2[-1L]
-
-.match.definition <- function(x, table,
-                              nomatch = NA_integer_, incomparables = NULL)
-{
-    if (!identical(nomatch, NA_integer_))
-        stop("'nomatch' arg is not supported")
-    msg <- c("match() between a ", class(x), " and a ", class(table),
-             " object is defunct.\nPlease use '",
-             "findOverlaps(x, table, select=\"first\")",
-             "' instead.")
-    .Defunct(msg=msg)  # deprecated in BioC 2.12, defunct in BioC 2.13
-}
-
-setMethods("match", .signatures3, .match.definition)
-
-setMethods("%in%", .signatures3, IRanges:::`.%in%.definition`)
-
-### The only reason for defining the methods below is to prevent the default
-### "findMatches" or "countMatches" methods to be called and return something
-### wrong (and the reason they would return something wrong is because they
-### are based on match() which does overlaps instead of equality).
-### TODO: Remove these methods in BioC 2.14 when the "match" methods for all
-### the signatures in '.signatures3' are gone.
-
-setMethods("findMatches", .signatures3,
-    function(x, table, select=c("all", "first", "last"), ...)
-    {
-        msg <- c("findMatches() between a ", class(x), " and a ",
-                 class(table), " object is not supported")
-        stop(msg)
-    }
-)
-
-setMethods("countMatches", .signatures3,
-    function(x, table, ...)
-    {
-        msg <- c("countMatches() between a ", class(x), " and a ",
-                 class(table), " object is not supported")
-        stop(msg)
-    }
-)
-
