@@ -3,7 +3,7 @@
 ###################################################
 ### code chunk number 1: style
 ###################################################
-BiocStyle::latex()
+BiocStyle::latex(use.unsrturl=FALSE)
 
 
 ###################################################
@@ -17,8 +17,6 @@ options("showTailLines" = 3)
 ###################################################
 ### code chunk number 3: load
 ###################################################
-library(GenomicRanges)
-library(Rsamtools)
 library(pasillaBamSubset)
 un1 <- untreated1_chr4() ## single-end reads
 
@@ -26,7 +24,7 @@ un1 <- untreated1_chr4() ## single-end reads
 ###################################################
 ### code chunk number 4: readGAlignments
 ###################################################
-un1 <- untreated1_chr4()
+library(GenomicAlignments)
 gal <- readGAlignments(un1)
 
 
@@ -102,8 +100,6 @@ cvg
 ###################################################
 ### code chunk number 13: load
 ###################################################
-library(GenomicRanges)
-library(Rsamtools)
 library(pasillaBamSubset)
 un1 <- untreated1_chr4() ## single-end records
 
@@ -118,6 +114,7 @@ exbygene <- exonsBy(TxDb.Dmelanogaster.UCSC.dm3.ensGene, "gene")
 ###################################################
 ### code chunk number 15: count_2
 ###################################################
+library(GenomicAlignments)
 se <- summarizeOverlaps(exbygene, un1, mode="IntersectionNotEmpty")
 
 
@@ -236,14 +233,14 @@ library(BSgenome.Hsapiens.UCSC.hg19)
 ###################################################
 ### code chunk number 32: trak_9
 ###################################################
-intron_seqs <- extractTranscriptsFromGenome(Hsapiens, intronsbytx)
+intron_seqs <- extractTranscriptSeqs(Hsapiens, intronsbytx)
 intron_seqs
 
 
 ###################################################
 ### code chunk number 33: trak_10
 ###################################################
-exon_seqs <- extractTranscriptsFromGenome(Hsapiens, exonsbytx)
+exon_seqs <- extractTranscriptSeqs(Hsapiens, exonsbytx)
 exon_seqs
 
 
@@ -328,9 +325,9 @@ genome <- BSgenome.Hsapiens.UCSC.hg19
 ###################################################
 ### code chunk number 44: cancer_11
 ###################################################
-threeUTR_seqs <- extractTranscriptsFromGenome(genome, threeUTR) 
-fiveUTR_seqs <- extractTranscriptsFromGenome(genome, fiveUTR) 
-cds_seqs <- extractTranscriptsFromGenome(genome, cds) 
+threeUTR_seqs <- extractTranscriptSeqs(genome, threeUTR) 
+fiveUTR_seqs <- extractTranscriptSeqs(genome, fiveUTR) 
+cds_seqs <- extractTranscriptSeqs(genome, cds) 
 
 
 ###################################################
@@ -342,10 +339,9 @@ cds_seqs
 ###################################################
 ### code chunk number 46: cancer_13
 ###################################################
-lst3 <- split(threeUTR_seqs, PartitioningByWidth(sum(map %in% names(threeUTR))))
-lst5 <- split(fiveUTR_seqs, PartitioningByWidth(sum(map %in% names(fiveUTR))))
-lstc <- split(cds_seqs, PartitioningByWidth(sum(map %in% names(cds))))
-names(lst3) <- names(lst5) <- names(lstc) <- names(map)
+lst3 <- relist(threeUTR_seqs, PartitioningByWidth(sum(map %in% names(threeUTR))))
+lst5 <- relist(fiveUTR_seqs, PartitioningByWidth(sum(map %in% names(fiveUTR))))
+lstc <- relist(cds_seqs, PartitioningByWidth(sum(map %in% names(cds))))
 
 
 ###################################################
